==> BestCrypt-2.0.7.tar.gz: BestCrypt-2.0.7/certificate/CertificateLib/symmetricAlgs/cSymAlg.h <==
/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2012 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: cSymAlg.h
//      Revision:
//      Created:
//      Description: Base class for all symmetric algorithms
//
/////////////////////////////////////////////////////////////////

#ifndef __cSymAlg__
#define __cSymAlg__

#include <stdlib.h>
#include <memory.h>
#include "../SMemory/smemory.h"
#include "ALG_DEF.H"

#include "../crtLibErrs.h"


class cSymAlg
{
  private:

	BYTE *m_iv;
	BYTE *m_prev;
	int  m_iv_offset;
  
  protected:

	static BYTE *safeAlloc( int size ) 	{ return (BYTE *)SMalloc( size ); }
	static void safeFree( BYTE *mem ) 	{ SFree( mem ); }

  public:

	typedef int typeProcessCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int  bfSize );

	typeProcessCBC *m_encryptCBC;
	typeProcessCBC *m_decryptCBC;

	void setBCDriverCompatible( bool bSet )
	{
		if ( bSet )
		{
			m_encryptCBC = encryptCBC_BCDriverCompatible;
			m_decryptCBC = decryptCBC_BCDriverCompatible;
		}
		else
		{
			m_encryptCBC = encryptCBC;
			m_decryptCBC = decryptCBC;
		}
	}


	cSymAlg();
	virtual ~cSymAlg();
#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	virtual BOOL getAlgConstants( int *blockSize, int *keySize ) = 0;

	// Encryption and decription in Electronic Code Book mode has to be implemented at child class

	virtual int encryptECB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		) = 0;
	

	virtual int decryptECB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		) = 0;

#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
	//
	//	Encryption and decription in Cipher Block Chaining mode
	//		Both function return sae_invalidBlockSize if buffer size 
	//		is not devided by algorithm block size.
	//

	int encryptCBC( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	int decryptCBC( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );

	static int encryptCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->encryptCBC( sync, inBf,  outBf,  bfSize ); }
	static int decryptCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->decryptCBC( sync, inBf,  outBf,  bfSize ); }

	//
	//	11.04.2013
	//
	// BCDriverCompatible
	//	In driver CAST and IDEA CBC mode processing functions treat input buffer as DWORD array and covert it to BIG-ENDIAN BYTE flow.
	//	For container compatibility we have to perform this operations.
	//	Use setBCDriverCompatible( true ) command for this class and functions will be changed automatically,
	//	by default sa_encryptCBC and sa_decryptCBC functions use original encryptCBC and decryptCBC, see cSymAlg() constructor.
	//
	int encryptCBC_BCDriverCompatible( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	int decryptCBC_BCDriverCompatible( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	static int encryptCBC_BCDriverCompatible( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->encryptCBC_BCDriverCompatible( sync, inBf,  outBf,  bfSize ); }
	static int decryptCBC_BCDriverCompatible( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->decryptCBC_BCDriverCompatible( sync, inBf,  outBf,  bfSize ); }

	//
	//	Encryption and decription in Cipher Feedback mode
	//		
	int initCFB( const BYTE *iv );
	int syncCFB(); // see the NOTE below

	int encryptCFB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);

	int decryptCFB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);
};


#endif // __cSymAlg__


/************* NOTE concerning syncCFB  ************************************

From:
Network Working Group
Request for Comments: 2440
Category: Standards Track
(RFC2440)

   The data is encrypted in CFB mode, with a CFB shift size equal to the
   cipher's block size.  The Initial Vector (IV) is specified as all
   zeros.  Instead of using an IV, OpenPGP prefixes a 10-octet string to
   the data before it is encrypted.  The first eight octets are random,
   and the 9th and 10th octets are copies of the 7th and 8th octets,
   respectively. After encrypting the first 10 octets, the CFB state is
   resynchronized if the cipher block size is 8 octets or less.  The
   last 8 octets of ciphertext are passed through the cipher and the
   block boundary is reset.

****************************************************************************/

==> BestCrypt-2.0.14.tar.gz: BestCrypt-2.0.14/certificate/CertificateLib/symmetricAlgs/cSymAlg.h <==
/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2012 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: cSymAlg.h
//      Revision:
//      Created:
//      Description: Base class for all symmetric algorithms
//
/////////////////////////////////////////////////////////////////

#ifndef __cSymAlg__
#define __cSymAlg__

#include <stdlib.h>
#include <memory.h>
#include "../SMemory/smemory.h"
#include "ALG_DEF.H"

#include "../crtLibErrs.h"


class cSymAlg
{
  private:

	BYTE *m_iv;
	BYTE *m_prev;
	int  m_iv_offset;
  
  protected:

	static BYTE *safeAlloc( int size ) 	{ return (BYTE *)SMalloc( size ); }
	static void safeFree( BYTE *mem ) 	{ SFree( mem ); }

  public:

	typedef int typeProcessCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int  bfSize );

	typeProcessCBC *m_encryptCBC;
	typeProcessCBC *m_decryptCBC;

	enum eCompatibleType { eOriginial, eCBCCompatible, eCFBCompatible };
	void setBCDriverCompatible( eCompatibleType eType )
	{
		switch ( eType )
		{
			case eCBCCompatible:
				m_encryptCBC = encryptCBC_BCDriverCompatible;
				m_decryptCBC = decryptCBC_BCDriverCompatible;
				break;
			case eCFBCompatible:
				m_encryptCBC = encryptCFB;
				m_decryptCBC = decryptCFB;
				break;
			default:
				m_encryptCBC = encryptCBC;
				m_decryptCBC = decryptCBC;
				break;
		}
	}


	cSymAlg();
	virtual ~cSymAlg();
#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	virtual BOOL getAlgConstants( int *blockSize, int *keySize ) = 0;

	// Encryption and decription in Electronic Code Book mode has to be implemented at child class

	virtual int encryptECB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		) = 0;
	

	virtual int decryptECB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		) = 0;

#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
	//
	//	Encryption and decription in Cipher Block Chaining mode
	//		Both function return sae_invalidBlockSize if buffer size 
	//		is not devided by algorithm block size.
	//

	int encryptCBC( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	int decryptCBC( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );

	static int encryptCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->encryptCBC( sync, inBf,  outBf,  bfSize ); }
	static int decryptCBC( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->decryptCBC( sync, inBf,  outBf,  bfSize ); }

	//
	//	11.04.2013
	//
	// BCDriverCompatible
	//	In driver CAST and IDEA CBC mode processing functions treat input buffer as DWORD array and covert it to BIG-ENDIAN BYTE flow.
	//	For container compatibility we have to perform this operations.
	//	Use setBCDriverCompatible( true ) command for this class and functions will be changed automatically,
	//	by default sa_encryptCBC and sa_decryptCBC functions use original encryptCBC and decryptCBC, see cSymAlg() constructor.
	//
	int encryptCBC_BCDriverCompatible( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	int decryptCBC_BCDriverCompatible( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	static int encryptCBC_BCDriverCompatible( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->encryptCBC_BCDriverCompatible( sync, inBf,  outBf,  bfSize ); }
	static int decryptCBC_BCDriverCompatible( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->decryptCBC_BCDriverCompatible( sync, inBf,  outBf,  bfSize ); }

	//
	//	Encryption and decription in Cipher Feedback mode
	//		
	int initCFB( const BYTE *iv );
	int syncCFB(); // see the NOTE below

	int encryptCFB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);

	int decryptCFB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);

	int encryptCFB( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	int decryptCFB( const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize );
	static int encryptCFB( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->encryptCFB( sync, inBf,  outBf,  bfSize ); }
	static int decryptCFB( cSymAlg *pThis, const BYTE *sync, const BYTE *inBf, BYTE *outBf, int bfSize ) { return pThis->decryptCFB( sync, inBf,  outBf,  bfSize ); }
};


#endif // __cSymAlg__


/************* NOTE concerning syncCFB  ************************************

From:
Network Working Group
Request for Comments: 2440
Category: Standards Track
(RFC2440)

   The data is encrypted in CFB mode, with a CFB shift size equal to the
   cipher's block size.  The Initial Vector (IV) is specified as all
   zeros.  Instead of using an IV, OpenPGP prefixes a 10-octet string to
   the data before it is encrypted.  The first eight octets are random,
   and the 9th and 10th octets are copies of the 7th and 8th octets,
   respectively. After encrypting the first 10 octets, the CFB state is
   resynchronized if the cipher block size is 8 octets or less.  The
   last 8 octets of ciphertext are passed through the cipher and the
   block boundary is reset.

****************************************************************************/
